// ==UserScript==
// @name        AccordionPrefaceAndPostface
// @namespace   https://ux.getuploader.com/narou/
// @description 小説家になろうの小説の前書き・後書き・ランキングタグ領域を折り畳む
// @author      No Name
// @match       *://ncode.syosetu.com/n*
// @match       *://novel18.syosetu.com/n*
// @version     1.1.1
// @grant       none
// ==/UserScript==

(function() {
	'use strict';

	//検証用
	const useConsole = false; //コンソールへの出力が必要ならtrueにする
	const writeLog = function(s) {
		if (useConsole) console.log("【小説の前書き・後書きを折り畳む】" + s);
	};

	const css = (function () {/***
		#novel_contents input[type="checkbox"].accordion { display:none }
		#novel_contents input[type="checkbox"].accordion + label {
			display:inline-block; padding:2px 0.5em; background-color:lightgray; border-radius:5px;
			cursor:pointer;
		}
		#novel_contents input[type="checkbox"].accordion + label.postface { margin-top:30px; margin-bottom:30px }
		#novel_contents input[type="checkbox"].accordion + label.rankingTag {
			display:block; width:90%; margin:20px auto auto auto;
		}
		#novel_contents input[type="checkbox"].accordion + label::after { content:"▼"; margin-left:5px }
		#novel_contents input[type="checkbox"].accordion + label + div {
			margin-top:0; max-height:0; border:none; padding:0; overflow-y:hidden;
		}
		#novel_contents input[type="checkbox"].accordion + label.rankingTag + div {
			width:90%; margin-left:auto; margin-right:auto;
		}
		//展開時の指定
		#novel_contents input[type="checkbox"].accordion:checked + label {
			background-color:black; color:white; border-radius: 5px 5px 0 0;
		}
		#novel_contents input[type="checkbox"].accordion:checked + label.postface { margin-bottom:0 }
		#novel_contents input[type="checkbox"].accordion:checked + label::after { content:"▲" }
		#novel_contents input[type="checkbox"].accordion:checked + label + div {
			max-height:300px; overflow-y:auto; transition:all 0.3s;
			padding:7px 1ex; border:1px solid darkgray;
			background-color:white; color:dimgray;
		}
		#novel_contents input[type="checkbox"].accordion:checked + label + div * { color:gray; }
	***/}).toString().replace(/\/\/.*\n/g, "").replace(/[\r\n\t]/g, "").split("***")[1];

	const define = {
		"preface": {
			"text": "前書き",
			"id": "forPreface",
			"active": true, //true:折り畳む false:折り畳まない
		},
		"postface": {
			"text": "後書き",
			"id": "forPostface",
			"active": true, //true:折り畳む false:折り畳まない
		},
		"rankingTag": {
			"text": "ランキングタグ",
			"id": "forRankingTag",
			"active": true, //true:折り畳む false:折り畳まない
		},
	};

	const insertAccordionHeader = function(e, defname) {
		if (!e) return;
		if (!e.textContent.replace(/\s/g,"")) {
			//実質的に空なら単純に非表示化
			e.style.display = "none";
			return;
		}
		const def = define[defname];
		if (!def || !def.active) return;
		writeLog(def.text);

		//checkbox
		const ckbox = document.createElement("input");
		ckbox.type = "checkbox";
		ckbox.id = def.id;
		ckbox.className = "accordion";
		e.parentNode.insertBefore(ckbox, e);

		//label
		const label = document.createElement("label");
		label.htmlFor = ckbox.id;
		label.className = defname;
		label.appendChild(document.createTextNode(def.text));
		e.parentNode.insertBefore(label, e);
	};

	const container = document.getElementById("novel_contents");
	if (!container) return;
	const preface = container.querySelector("#novel_color > #novel_p");
	const postface = container.querySelector("#novel_color > #novel_a");
	const ranking = container.querySelector(".novelrankingtag, .rankingtag_margin"); //後者はスマホ用
	if (preface || postface || ranking) {
		const style = document.createElement("style");
		style.innerHTML = css;
		document.head.appendChild(style);
		insertAccordionHeader(preface, "preface");
		insertAccordionHeader(postface, "postface");
		insertAccordionHeader(ranking, "rankingTag");
	}

})();
