// ==UserScript==
// @name        NarouNGTitle
// @namespace   https://ux.getuploader.com/narou/
// @description 指定した正規表現パターンに合致するタイトルの小説を非表示にする
// @author      No Name
// @match       *://syosetu.com/
// @match       *://syosetu.com/pickup/list/*
// @match       *://yomou.syosetu.com/
// @match       *://yomou.syosetu.com/search.php*
// @match       *://yomou.syosetu.com/rank/*
// @match       *://yomou.syosetu.com/*list/*list/*
// @version     1.0.2
// @grant       none
// ==/UserScript==

(function() {
	'use strict';

//非表示対象タイトルの指定
const ngTitlePatternMap = [
//サンプル
["ダイエット業者",/１ヶ月で１２キロ痩せました/],
];

	//////////// 検証用 ////////////
	const useConsole = false; //true:コンソールに出力する、false:コンソールに出力しない
	const writeLog = function(s) {
		if (useConsole) console.log("【指定したタイトルの作品を非表示化】" + s);
	};

	//////////// 共通処理: 表示非表示切り替え ////////////
	const classNameHideWriter = "hideWriter";
	const classNameHide = "hideTitle";
	const classNameShow = "temporarilyTitle";
	const msgToShow = "クリックで非表示を一時解除する";
	const msgRetryHide = "ダブルクリックで再度非表示にする"; //a要素クリック時に非表示にしないため
	const showNovelBlock = function() {
		this.classList.remove(classNameHide);
		this.classList.add(classNameShow);
		this.removeEventListener("click", showNovelBlock, false);
		this.addEventListener("dblclick", hideNovelBlock, false);
		this.title = msgRetryHide;
	};
	const hideNovelBlock = function() {
		this.classList.remove(classNameShow);
		this.classList.add(classNameHide);
		this.removeEventListener("dblclick", hideNovelBlock, false);
		this.addEventListener("click", showNovelBlock, false);
		this.title = msgToShow;
	};
	const toggleCursorCss = "_selector_." + classNameHide + ",_selector_." + classNameShow + "{ cursor:pointer }";

	//////////// 共通処理 ////////////
	const attName = "data-label";
	const extractCss = function(f) {
		return f.toString().replace(/\/\/.*\n/g, "").replace(/[\r\n\t]/g, "").split("***")[1];
	};
	const addCss = function(def) {
		if (def.css) {
			const style = document.createElement("style");
			if (def.toggle) style.innerHTML = toggleCursorCss.replace(/_selector_/g, def.selector.block);
			style.innerHTML += extractCss(def.css).replace(/_attName_/g, attName);
			document.head.appendChild(style);
		}
	};
	const setCustomAttribute = function(e, label) {
		e.setAttribute(attName, label);
		writeLog(label.substr(0,40));
	};

	//////////// ページ毎の処理本体 ////////////
	const なろうトップ処理 = function(def) { //コンテナ要素がないので、単純に非表示にする
		addCss(def);
		for (let a of document.querySelectorAll(def.selector.block)) {
			const span = a.querySelector(def.selector.title);
			if (span && span.textContent) {
				const content = span.textContent;
				for (let [label, rule] of ngTitlePatternMap) {
					if (content.match(rule)) {
						a.classList.add(classNameHide);
						writeLog(label.substr(0,40));
						break;
					}
				}
			}
		}
	};
	const 読もうトップ処理 = function(def) { //コンテナ要素の全子要素を非表示化、代替でラベル表示
		addCss(def);
		for (let e of document.querySelectorAll(def.selector.block)) {
			const a = e.querySelector(def.selector.title);
			if (a && a.textContent) {
				const content = a.textContent;
				for (let [label, rule] of ngTitlePatternMap) {
					if (content.match(rule)) {
						e.classList.add(classNameHide);
						setCustomAttribute(e, label);
						break;
					}
				}
			}
		}
	};
	const 一覧系共通処理 = function(def) { //コンテナ要素の全子要素を非表示化、代替でラベル表示、非表示／再表示でトグル可
		addCss(def);
		for (let e of document.querySelectorAll(def.selector.block)) {
			if (e.classList.contains(classNameHideWriter)) continue; //非表示対象作者なら何もしない
			const a = e.querySelector(def.selector.title);
			if (a && a.textContent) {
				const content = a.textContent;
				for (let [label, rule] of ngTitlePatternMap) {
					if (content.match(rule)) {
						e.classList.add(classNameHide);
						e.addEventListener("click", showNovelBlock, false);
						e.title = msgToShow;
						setCustomAttribute(e, label);
						break;
					}
				}
			}
		}
	};

	//////////// 処理定義 ////////////
	const define = {
		"top": {
			"path": {
				"top": {
					"caption": "なろうトップ",
					"rule": /^$/,
					"func": なろうトップ処理,
					"selector": {
						"block": ".in_box > a",
						"title": "span",
					},
					"css": (function () {/***
						//非表示時の設定
						.in_box > a.hideTitle { display:none; }
					***/}),
				},
				"pickup": {
					"caption": "ピックアップリスト",
					"rule": /^pickup\/list\//,
					"func": 一覧系共通処理,
					"selector": {
						"block": ".trackback_list",
						"title": "div.trackback_listdiv > a",
					},
					"toggle": true,
					"css": (function () {/***
						//非表示時の設定
						.trackback_list.hideTitle { border:1px dashed lightgray; }
						.trackback_list.hideTitle * { display:none; }
						.trackback_list.hideTitle::after {
							content:"〔非表示対象タイトル〕" attr(_attName_); color:lightgray;
						}
						//再表示時の設定
						.trackback_list.temporarilyTitle { border:1px solid DeepPink !important }
						.trackback_list.temporarilyTitle::before {
							content: "一時非表示解除中（ダブルクリックで再度非表示化）"; color:white;
							display:inline-block; padding:2px .5em; margin-right:.8em; background-color:DeepPink;
						}
					***/}),
				},
			},
		},
		"yomou": {
			"path": {
				"top": {
					"caption": "読もうトップ",
					"rule": /^$/,
					"func": 読もうトップ処理,
					"selector": {
						"block": ".in_box > ul.list_p > li",
						"title": ".list_p_title",
					},
					"css": (function () {/***
						//非表示時の設定
						.in_box > ul.list_p > li.hideTitle { border:none; }
						.in_box > ul.list_p > li.hideTitle * { display:none; }
						.in_box > ul.list_p > li.hideTitle::after {
							content:"〔非表示対象タイトル〕" attr(_attName_); color:lightgray;
						}
					***/}),
				},
				"search": {
					"caption": "検索結果",
					"rule": /^search\.php$/,
					"func": 一覧系共通処理,
					"selector": {
						"block": ".searchkekka_box",
						"title": "div.novel_h > a.tl",
					},
					"toggle": true,
					"css": (function () {/***
						//非表示時の設定
						.searchkekka_box.hideTitle { border:1px dashed lightgray; }
						.searchkekka_box.hideTitle * { display:none; }
						.searchkekka_box.hideTitle::after {
							content:"〔非表示対象タイトル〕" attr(_attName_); color:lightgray;
						}
						//再表示時の設定
						.searchkekka_box.temporarilyTitle { border:1px solid DeepPink; }
						.searchkekka_box.temporarilyTitle::before {
							content:"一時非表示解除中（ダブルクリックで再度非表示化）"; color:white;
							display:inline-block; padding:2px .5em; margin-right:.8em; background-color:DeepPink;
						}
					***/}),
				},
				"rankgenretop": {
					"caption": "ランキングトップ（ジャンル別）",
					"rule": /^rank\/(?:genre)top\//,
					"func": 一覧系共通処理,
					"selector": {
						"block": ".genreranking_topbox > ul > li",
						"title": "a.tl",
					},
					"toggle": true,
					"css": (function () {/***
						//非表示時の設定
						.genreranking_topbox > ul > li.hideTitle { border:1px dashed lightgray; }
						.genreranking_topbox > ul > li.hideTitle * { display:none; }
						.genreranking_topbox > ul > li.hideTitle::after {
							content:"〔非表示〕" attr(_attName_); color:lightgray;
						}
						//再表示時の設定
						.genreranking_topbox > ul > li.temporarilyTitle { border:1px solid DeepPink; }
						.genreranking_topbox > ul > li.temporarilyTitle::before {
							content:"一時解除"; color:white; background-color:DeepPink;
							display:inline-block; padding:2px .5em; margin-right:.1em;
						}
					***/}),
				},
				"ranktop": {
					"caption": "ランキングトップ（総合、異世界転生/転移）",
					"rule": /^rank\/(?:isekai)?top\//,
					"func": 一覧系共通処理,
					"selector": {
						"block": ".ranking_top5box > ul > li",
						"title": "a.tl",
					},
					"toggle": true,
					"css": (function () {/***
						//非表示時の設定
						.ranking_top5box > ul > li.hideTitle { border:1px dashed lightgray; }
						.ranking_top5box > ul > li.hideTitle * { display:none; }
						.ranking_top5box > ul > li.hideTitle::after {
							content:"〔非表示対象タイトル〕" attr(_attName_); color:lightgray;
						}
						//再表示時の設定
						.ranking_top5box > ul > li.temporarilyTitle { border:1px solid DeepPink; }
						.ranking_top5box > ul > li.temporarilyTitle::before {
							content:"一時非表示解除中（ダブルクリックで再度非表示化）"; color:white;
							display:inline-block; padding:2px .5em; margin-right:.8em; background-color:DeepPink;
						}
					***/}),
				},
				"ranklist": {
					"caption": "ランキング一覧",
					"rule": /^rank\/(?:[a-z]+)?list\//,
					"func": 一覧系共通処理,
					"selector": {
						"block": ".ranking_list",
						"title": ".rank_h > a.tl",
					},
					"toggle": true,
					"css": (function () {/***
						//非表示時の設定
						.ranking_list.hideTitle { border:1px dashed lightgray; }
						.ranking_list.hideTitle * { display:none }
						.ranking_list.hideTitle::after {
							content:"〔非表示対象タイトル〕" attr(_attName_); color:lightgray;
						}
						//再表示時の設定
						.ranking_list.temporarilyTitle { border:1px solid DeepPink }
						.ranking_list.temporarilyTitle::before {
							content:"一時非表示解除中（ダブルクリックで再度非表示化）"; color:white;
							display:inline-block; padding:2px .5em; margin-right:.8em; background-color:DeepPink;
						}
					***/}),
				},
				"nopointlist": {
					"caption": "未評価作品一覧、未感想作品一覧",
					"rule": /^nolist\/no(?:point|impression)list\//,
					"func": 一覧系共通処理,
					"selector": {
						"block": ".no_list",
						"title": ".no_list_head > a.novel_title",
					},
					"toggle": true,
					"css": (function () {/***
						//非表示時の設定
						.no_list.hideTitle * { display:none; }
						.no_list.hideTitle::after {
							content:"〔非表示対象タイトル〕" attr(_attName_); color:lightgray;
						}
						//再表示時の設定
						.no_list.temporarilyTitle { border:1px solid DeepPink; }
						.no_list.temporarilyTitle::before {
							content:"一時非表示解除中（ダブルクリックで再度非表示化）"; color:white;
							display:inline-block; padding:2px .5em; margin-right:.8em; background-color:DeepPink;
						}
					***/}),
				},
			},
		},
	};

	//////////// メイン処理 ////////////
	const getDefineByUrl = function() {
		const host = location.hostname.toLowerCase();
		const path = location.pathname.toLowerCase().replace(/^\//, "");
		const subdomain = host.match(/^([a-z\d]+)\.syosetu\.com/) ? RegExp.$1 : "top";
		const item = define[subdomain];
		if (item) {
			for (let kind in item.path) {
				const o = item.path[kind];
				if (path.match(o.rule)) return { def:o, param:RegExp.$1 };
			}
		}
	};
	const defContainer = getDefineByUrl();
	if (defContainer) {
		const def = defContainer.def;
		if (def.func) {
			writeLog(def.caption);
			def.func(def, defContainer.param);
		} else {
			writeLog("処理未登録");
		}
	} else {
		writeLog("対象外ページ");
	}
})();
