// ==UserScript==
// @name        NarouNGWriter
// @namespace   https://ux.getuploader.com/narou/
// @description 小説家になろうの一覧系ページで指定した作者の小説を非表示にする
// @author      No Name
// @match       *://syosetu.com/pickup/list/*
// @match       *://syosetu.com/searchuser/search/*
// @match       *://yomou.syosetu.com/search.php*
// @match       *://yomou.syosetu.com/search/search/*
// @match       *://yomou.syosetu.com/rank/*
// @match       *://yomou.syosetu.com/*list/*list/*
// @match       *://mypage.syosetu.com/*
// @match       *://xmypage.syosetu.com/*
// @match       *://ncode.syosetu.com/n*
// @match       *://ncode.syosetu.com/s*
// @match       *://novel18.syosetu.com/n*
// @match       *://novel18.syosetu.com/xs*
// @version     1.8
// @grant       none
// ==/UserScript==

(function() {
	'use strict';

//非表示対象作者の指定
const ngWriterList = {
//サンプル（ソート前の書籍化経験者一覧から先頭5件を拝借して改変）
'622537' : '説明文例：作者名＝瀬戸メグル',
'994271' : '説明文例：作者名＝月島秀一',
'224211' : '',//説明省略例（空文字）
'602510' : null,//説明省略例（null）
'258229' : '説明文例：作者名＝ピチ＆メル/三丘　洋',
};

	//////////// 検証用 ////////////
	const useConsole = false; //コンソールへの出力が必要ならtrueにする
	const writeLog = function(s) {
		if (useConsole) console.log("【指定した作者の作品を非表示化】" + s);
	};

	//////////// 共通処理: 表示非表示切り替え ////////////
	const classNameHide = "hideWriter";
	const classNameShow = "temporarily";
	const msgToShow = "クリックで非表示を一時解除する";
	const msgRetryHide = "ダブルクリックで再度非表示にする"; //a要素クリック時に非表示にしないため
	const showNovelBlock = function() {
		this.classList.remove(classNameHide);
		this.classList.add(classNameShow);
		this.removeEventListener("click", showNovelBlock, false);
		this.addEventListener("dblclick", hideNovelBlock, false);
		this.title = msgRetryHide;
	};
	const hideNovelBlock = function() {
		this.classList.remove(classNameShow);
		this.classList.add(classNameHide);
		this.removeEventListener("dblclick", hideNovelBlock, false);
		this.addEventListener("click", showNovelBlock, false);
		this.title = msgToShow;
	};
	const toggleCursorCss = "_selector_." + classNameHide + ",_selector_." + classNameShow + "{ cursor:pointer }";

	//////////// 共通処理 ////////////
	const attName = "data-writer";
	const extractCss = function(f) {
		if (!f) return "";
		return f.toString().replace(/\/\/.*\n/g, "").replace(/[\r\n\t]/g, "").split("***")[1];
	};
	const addCss = function(def) {
		if (Array.isArray(def.css) && def.css.length) {
			const style = document.createElement("style");
			if (def.toggle) style.innerHTML = toggleCursorCss.replace(/_selector_/g, def.selector);
			for (let i=0, len=def.css.length ; i<len ; i++) {
				style.innerHTML += extractCss(defineCss[def.css[i]]).replace(/_attName_/g, attName);
			}
			document.head.appendChild(style);
		}
	};
	const setCustomAttribute = function(e, id, writer, note) {
		const s = id + "（" + writer + "）" + (note ? note : "");
		e.setAttribute(attName, s);
		writeLog(s.substr(0,40));
	};

	//////////// 整数->XID変換 ////////////
	const alphabet = "abcdefghijklmnopqrstuvwxyz";
	const alphalen = alphabet.length;
	const maxXidNum = 9999;
	const padZero = function(n) {
		return ("000" + n).slice(-4);
	};
	const intToXid = function(n) {
		const array = [];
		let v = n / maxXidNum | 0;
		const code = n % maxXidNum || maxXidNum;
		if (code == maxXidNum) v -= 1;
		do {
			array.push(alphabet.substr(v % alphalen, 1));
			v = v / alphalen | 0;
		} while(v);
		return "x" + padZero(code) + array.reverse().join("");
	};

	//////////// ページ毎の処理本体 ////////////
	const ユーザーID抽出用定義 = {
		selector: 'a[href*="mypage"]',
		rule: /x?mypage\.syosetu\.com\/(\d+|x\d{4}[a-z]+)\//,
		ruleN: /mypage\.syosetu\.com\/(\d+)\//,
		ruleX: /xmypage\.syosetu\.com\/(x\d{4}[a-z]+)\//,
	};
	const マイページ共通処理 = function(def, id) {
		const note = ngWriterList[id];
		if (note === undefined) {
			return false;
		} else {
			addCss(def);
			if (note) {
				const e = document.querySelector(def.selector);
				if (e) e.setAttribute(attName, note);
			}
			return true;
		}
	};
	const Ｘマイページ処理 = function(def, xid) {
		//XIDの処理
		const isTargetMypage = マイページ共通処理(def, xid);
		//通常マイページへのリンク処理
		const a = document.querySelector(".profile .islink " + ユーザーID抽出用定義.selector);
		if (a && a.href.match(ユーザーID抽出用定義.ruleN)) {
			const note = ngWriterList[RegExp.$1]; //ユーザーIDは別途判定
			if (note !== undefined) {
				a.classList.add(classNameHide);
				if (note) a.title = note;
				if (!isTargetMypage) addCss(def); //css未追加なら追加
			}
		}
	};
	const Ｘユーザー処理 = function(def, nXid) {
		マイページ共通処理(def, intToXid(nXid));
	};
	const 小説ページ処理 = function(def) {
		const a = document.querySelector("#novel_footer > ul.undernavi > li > " + ユーザーID抽出用定義.selector);
		if (a && a.href.match(ユーザーID抽出用定義.rule)) {
			if (ngWriterList[RegExp.$1] !== undefined) addCss(def);
		}
	};
	const シリーズページ処理 = function(def) {
		const a = document.querySelector("#novel_footernavi ul.undernavi > li > " + ユーザーID抽出用定義.selector);
		if (a && a.href.match(ユーザーID抽出用定義.rule)) {
			if (ngWriterList[RegExp.$1] !== undefined) addCss(def);
		}
	};
	const 検索結果タイトル共通処理 = function(def) { //小説タイトルをanchorのtitle属性に格納
		if ("検索結果" === def.caption || "ランキング一覧" === def.caption) {
			const anchors = document.querySelectorAll(def.selector + " a.tl");
			for (let i=0, len=anchors.length ; i<len ; i++) {
				const a = anchors[i];
				a.title = a.firstChild.nodeValue;
			}
		}
	};
	const 除算 = function(int, div) {
		const rem = int % div;
		return { quot:(int-rem)/div, rem:rem };
	};
	const 読了時間換算処理 = function(stime, slen) {
		let time = null;
		if (stime) {
			time = Math.floor(stime.replace(/,/, ""));
		} else if (slen) {
			time = Math.ceil(slen.replace(/,/, "") / 500);
		} else {
			return null;
		}
		let res = 除算(time, 60);
		const mn = res.rem;
		res = 除算(res.quot, 24);
		const hh = res.rem;
		const dd = res.quot;
		let text = "";
		if (dd) text += dd + "日";
		if (hh) text += hh + "時間";
		if (mn) text += mn + "分";
		if (stime && text === stime + "分") return null;
		return `読了時間：約${text}（${slen}文字）`;
	};
	const 検索結果用読了時間換算処理 = function(e) {
		const spans = e.querySelectorAll("span");
		for (let i=0, len=spans.length ; i<len ; i++) {
			const span = spans[i];
			if (span.textContent.match(/読了時間：約([\d,]+)分（([\d,]+)文字）/)) {
				const stime = RegExp.$1;
				const slen = RegExp.$2;
				const times = 読了時間換算処理(stime, slen);
				if (times) span.innerHTML = times;
			}
		}
	};
	const 検索結果用各種算出処理 = function(def) {
		const elist = document.querySelectorAll(def.selector + " table tr td:last-child");
		for (let i=0, len=elist.length ; i<len ; i++) {
			const e = elist[i];
			let num = 0;
			let point = 0;
			if (e.textContent.match(/評価人数：\s*([\d,]+)\s*人/)) num = Math.floor(RegExp.$1.replace(/,/g, ""));
			if (e.textContent.match(/評価ポイント：\s*([\d,]+)\s*pt/)) point = Math.floor(RegExp.$1.replace(/,/g, ""));
			const span = document.createElement("span");
			const ave = num && point ? (Math.round(point/num*100)/100) : 0;
			span.innerHTML = "評価平均： " + ave + " pt";
			span.classList.add("average");
			span.classList.add("marginleft");
			e.appendChild(span);
			検索結果用読了時間換算処理(e);
		}
	};
	const 一覧系共通処理 = function(def) {
		addCss(def);
		const elist = document.querySelectorAll(def.selector);
		for (let i=0, len=elist.length ; i<len ; i++) {
			const e = elist[i];
			const a = e.querySelector(ユーザーID抽出用定義.selector);
 			if (a && a.href.match(ユーザーID抽出用定義.ruleN)) {
				const id = RegExp.$1;
				const note = ngWriterList[id];
				if (note !== undefined) {
					e.classList.add(classNameHide);
					if (def.toggle) { //表示非表示をトグルさせる場合
						e.addEventListener("click", showNovelBlock, false);
						e.title = msgToShow;
						setCustomAttribute(e, id, a.textContent, note);
					} else {
						if (note) e.title = note;
					}
				}
			}
		}
		検索結果タイトル共通処理(def);
	};
	const ジャンル別ランキングトップ処理 = function(def) {
		addCss(def);
		const elist = document.querySelectorAll(def.selector);
		for (let i=0, len=elist.length ; i<len ; i++) {
			const e = elist[i];
			const a = e.querySelector(ユーザーID抽出用定義.selector);
			if (a && a.href.match(ユーザーID抽出用定義.ruleN)) {
				const id = RegExp.$1;
				const note = ngWriterList[id];
				if (note !== undefined) {
					e.classList.add(classNameHide);
					e.addEventListener("click", showNovelBlock, false);
					e.title = msgToShow;
					setCustomAttribute(e, id, a.textContent);
				}
			}
		}
	};
	const text要素の空行削除処理 = function(e) {
		//冒頭は全角空白以外のホワイトスペース削除、末尾は全角空白とホワイトスペース削除
		//途中の行は複数の連続した改行を一個に集約
		e.nodeValue = e.nodeValue.replace(/^[ \r\n]+|[　\s]+$/,"").replace(/[\r\n]+/g,"\n");
	};
	const ランキング用読了時間整形処理 = function(def) {
		const spans = document.querySelectorAll("table.rank_table td span");
		for (let i=0, len=spans.length ; i<len ; i++) {
			const span = spans[i];
			if (span.textContent.match(/([\d,]+)文字/)) {
				span.innerHTML = 読了時間換算処理(null, RegExp.$1);
			}
		}
	};
	const ランキング処理 = function(def) {
		//tdのあらすじを高さ制限するためdivに格納
		const elist = document.querySelectorAll("td.ex");
		for (let i=0, len=elist.length ; i<len ; i++) {
			const e = elist[i];
			const div = document.createElement("div");
			const child = e.firstChild;
			if (3 === child.nodeType) {
				text要素の空行削除処理(child);
				div.appendChild(child);
			} else {
				while (e.firstChild) div.appendChild(e.firstChild);
			}
			e.appendChild(div);
		}
		一覧系共通処理(def);
		ランキング用読了時間整形処理(def);
	};

	//////////// CSS定義 ////////////
	const defineCss = {
		/*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
		 *
		 *   なろうのみ
		 *
		 *-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/
		//--------------------------------------ピックアップリスト
		"pickup": (function () {/***
			//非表示再表示の設定
			.trackback_list.hideWriter { border:1px dashed lightgray; }
			.trackback_list.hideWriter * { display:none; }
			.trackback_list.hideWriter::after {
				content:"〔非表示対象作者〕" attr(_attName_); color:lightgray;
			}
			.trackback_list.temporarily { border:1px solid red !important }
			.trackback_list.temporarily::before {
				content: "一時非表示解除中（ダブルクリックで再度非表示化）"; color:white; background-color:red;
				display:inline-block; padding:2px .5em; margin-right:.8em;
			}
		***/}),
		//--------------------------------------ユーザ検索
		"searchuser": (function () {/***
			//非表示対象作者にマーキング
			.main2_box > div.hideWriter::before {
				content:"一覧内作品非表示"; font-size:80%; color:white; background-color:red;
				display:inline-block; padding:2px 5px; line-height:100%;
			}
		***/}),
		//--------------------------------------検索結果
		"search": (function () {/***
			//小説情報
			.searchkekka_box { font-size:13px; }
			//リンク
			.searchkekka_box a { color:#0066cc; }
			.searchkekka_box a:visited { color:#7e007e; }
			.searchkekka_box a:hover,
			.searchkekka_box a:focus { color:red; opacity:1; text-decoration:underline; }
			//タイトル改行なし、はみ出し時省略、矩形付加
			.searchkekka_box > div.novel_h {
				white-space:nowrap; overflow-x:hidden; width:99%; font-size:12px;
				padding:1px 0px 1px 7px; border:1px solid #80808066; background-color:#e5d0bf;
			}
			//タイトルリンク
			.searchkekka_box > div.novel_h > a.tl {
				text-decoration:none; color:black; text-shadow:0px 0px 3px white;
			}
			.searchkekka_box > div.novel_h > a.tl:visited { color:#886436; }
			.searchkekka_box > div.novel_h > a.tl:hover,
			.searchkekka_box > div.novel_h > a.tl:focus { opacity:1; text-decoration:underline; }
			//小説内容の幅を100%に
			.searchkekka_box > table { width:100%; }
			//あらすじを高さ制限、文字色と背景色設定
			.searchkekka_box > table td div.ex {
				max-height:110px; overflow-y:auto; padding:2px 5px; font-size:103%;
				background-color:#fafaec; color:black; border:1px solid #80808022;
			}
			.searchkekka_box span.average {
				padding-left:5px; padding-right:5px; color:#ff6600; border:1px solid #ff6600;
			}
			//非表示再表示の設定
			.searchkekka_box.hideWriter { border:1px dashed lightgray; }
			.searchkekka_box.hideWriter * { display:none; }
			.searchkekka_box.hideWriter::after {
				content:"〔非表示対象作者〕" attr(_attName_); color:lightgray;
			}
			.searchkekka_box.temporarily { border:1px solid red; }
			.searchkekka_box.temporarily::before {
				content: "一時非表示解除中（ダブルクリックで再度非表示化）"; color:white; background-color:red;
				display:inline-block; padding:2px .5em; margin-right:.8em;
			}
		***/}),
		//--------------------------------------ランキングトップ（ジャンル別）
		"rankgenretop": (function () {/***
			//非表示再表示の設定
			.genreranking_topbox > ul > li.hideWriter { border:1px dashed lightgray; }
			.genreranking_topbox > ul > li.hideWriter * { display:none; }
			.genreranking_topbox > ul > li.hideWriter::after {
				content:"〔非表示〕" attr(_attName_); color:lightgray;
			}
			.genreranking_topbox > ul > li.temporarily { border:1px solid red; }
			.genreranking_topbox > ul > li.temporarily::before {
				content:"一時解除"; color:white; background-color:red;
				display:inline-block; padding:2px .5em; margin-right:.1em;
			}
			//タイトルリンク
			.genreranking_topbox > ul > li > a.tl:visited { color:#800; }
		***/}),
		//--------------------------------------ランキングトップ（総合、異世界転生/転移）
		"ranktop": (function () {/***
			//非表示再表示の設定
			.ranking_top5box > ul > li.hideWriter { border:1px dashed lightgray; }
			.ranking_top5box > ul > li.hideWriter * { display:none; }
			.ranking_top5box > ul > li.hideWriter::after {
				content:"〔非表示対象作者〕" attr(_attName_); color:lightgray;
			}
			.ranking_top5box > ul > li.temporarily { border:1px solid red; }
			.ranking_top5box > ul > li.temporarily::before {
				content:"一時非表示解除中（ダブルクリックで再度非表示化）"; color:white; background-color:red;
				display:inline-block; padding:2px .5em; margin-right:.8em;
			}
			//タイトルリンク
			.ranking_top5box > ul > li > a.tl:visited { color:#800; }
		***/}),
		//--------------------------------------ランキング一覧
		"ranklist": (function () {/***
			//リンク
			.ranking_list a { color:#0066cc; }
			.ranking_list a:visited { color:#7e007e; }
			.ranking_list a:hover,
			.ranking_list a:focus { color:red; opacity:1; text-decoration:underline; }
			//タイトル改行なし、はみ出し時省略、矩形付加
			.ranking_list > div.rank_h {
				white-space:nowrap; overflow-x:hidden; width:98%;
				padding:1px 0px 1px 7px; border:1px solid #80808066; background-color:#e5d0bf;
			}
			.deadlink {
				padding:3px 0px 3px 7px; width:98%;
				margin:0 0 30px 0; font-size:100%; font-weight:normal; border:1px solid lightgray;
			}
			.deadlink .ranking_number { width:auto; margin-right:.3em; color:#008b8b; }
			.ranking_list > div.rank_h > .ranking_number {
				width:auto; margin-right:.3em; color:#008b8b;
				text-shadow:1px 1px 0px white,-1px 1px 0px white,1px -1px 0px white,-1px -1px 0px white,
						1px 0px 0px white,0px 1px 0px white,-1px 0px 0px white,0px -1px 0px white;
			}
			//タイトルリンク
			.ranking_list > div.rank_h { font-size: 12px; }
			.ranking_list > div.rank_h > a.tl {
				text-decoration:none; color:black; text-shadow:0px 0px 3px white;
			}
			.ranking_list > div.rank_h > a.tl:visited { color:#886436; }
			.ranking_list > div.rank_h > a.tl:hover,
			.ranking_list > div.rank_h > a.tl:focus { opacity:1; text-decoration:underline; }
			//小説情報
			.ranking_list > table.rank_table { font-size:13px; }
			//ポイント情報
			.ranking_list > table.rank_table .point { font-size:13px !important; }
			//あらすじを高さ制限、文字色と背景色設定
			.ranking_list > table.rank_table td.ex  { font-size:100%; }
			.ranking_list > table.rank_table td.ex > div {
				max-height:123px; overflow-y:auto; padding:2px 5px; border:1px solid #80808022;
				padding:2px 5px; background-color:#fafaec; color:black; line-height:170%; white-space:pre-wrap;
			}
			//非表示再表示の設定
			.ranking_list.hideWriter { border:1px dashed lightgray; }
			.ranking_list.hideWriter * { display:none }
			.ranking_list.hideWriter::after {
				content:"〔非表示対象作者〕" attr(_attName_); color:lightgray;
			}
			.ranking_list.temporarily { border:1px solid red }
			.ranking_list.temporarily::before {
				content:"一時非表示解除中（ダブルクリックで再度非表示化）"; color:white; background-color:red;
				display:inline-block; padding:2px .5em; margin-right:.8em;
			}
		***/}),
		//--------------------------------------未評価作品一覧、未感想作品一覧
		"nopointlist": (function () {/***
			//非表示再表示の設定
			.no_list.hideWriter * { display:none; }
			.no_list.hideWriter::after {
				content:"〔非表示対象作者〕" attr(_attName_); color:lightgray;
			}
			.no_list.temporarily { border:1px solid red; }
			.no_list.temporarily::before {
				content:"一時非表示解除中（ダブルクリックで再度非表示化）"; color:white; background-color:red;
				display:inline-block; padding:2px .5em; margin-right:.8em;
			}
			//タイトルリンク
			.no_list a.novel_title { color:red; }
			.no_list a.novel_title:visited { color:#800; }
		***/}),
		//--------------------------------------平均点順作者一覧
		"avgranklist": (function () {/***
			//非表示対象作者にマーキング
			.review.hideWriter span.username::before {
				content:"一覧内作品非表示"; color:white; background-color:red;
				display:inline-block; padding:2px 5px; margin-right:.8em; line-height:100%;
			}
		***/}),
		/*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
		 *
		 *   マイページ
		 *
		 *-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/
		//なろう：マイページ共通
		"mypage": (function () {/***
			#anotheruser_info > h1::before {
				content:"一覧内作品非表示"; color:white; background-color:red;
				display:inline-block; padding:2px 5px; margin-right:.8em; line-height:100%;
			}
			#anotheruser_info > h1::after {
				content:attr(_attName_); color:red;
				display:block; font-weight:normal; font-size:13px; line-height:140%;
			}
		***/}),
		//18禁：Ｘマイページ共通
		"xmypage": (function () {/***
			#header > h1::before {
				content:"NG作者"; color:white; background-color:red;
				display:inline-block; padding:2px 5px; margin-right:.8em; line-height:100%;
			}
			#header > h1::after {
				content:attr(_attName_); color:gold;
				display:block; font-weight:normal; font-size:13px; line-height:140%;
			}
		***/}),
		//18禁：Ｘマイページトップ
		"xmypage_top": (function () {/***
			//通常マイページへのリンクにマーキング(ここが該当したらxidが未登録でもヘッダーにも"NG作者"がつく)
			.profile .islink a.hideWriter::after {
				content:"NG作者"; color:white; background-color:red;
				display:inline-block; padding:2px 4px; margin-left:.5em; line-height:100%;
			}
		***/}),
		/*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
		 *
		 *   小説ページ
		 *
		 *-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/
		//-------------------連載小説目次または短編
		//なろう＆18禁共通
		"ncode_top": (function () {/***
			#novel_color::before {
				color:white; background-color:red; border:1px solid white;
				font-weight:normal; padding:2px 5px; line-height:140%; display:inline-block;
			}
		***/}),
		//なろう
		"ncode_top_content": (function () {/***
			#novel_color::before { content:"一覧内非表示作品"; }
		***/}),
		//18禁
		"novel18_top_content": (function () {/***
			#novel_color::before { content:"NG作者作品"; }
		***/}),
		//-------------------連載小説各話
		//なろう＆18禁共通
		"ncode_novel": (function () {/***
			#container > .contents1::before {
				color:white; background-color:red; border:1px solid white;
				padding:2px 5px; font-size:12px; line-height:120%; display:inline-block;
			}
		***/}),
		//なろう
		"ncode_novel_content": (function () {/***
			#container > .contents1::before { content:"一覧内非表示作品"; }
		***/}),
		//18禁
		"novel18_novel_content": (function () {/***
			#container > .contents1::before { content:"NG作者作品"; }
		***/}),
		/*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
		 *
		 *   小説情報ページ
		 *
		 *-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/
		//なろう＆18禁共通
		"ncode_infotop": (function () {/***
			#ncode::before {
				color:white; background-color:red; display:inline-block;
				padding:2px 5px; margin-right:.5em; font-size:12px; line-height:120%;
			}
		***/}),
		//なろう
		"ncode_infotop_content": (function () {/***
			#ncode::before { content:"一覧内非表示作品"; }
		***/}),
		//18禁
		"novel18_infotop_content": (function () {/***
			#ncode::before { content:"NG作者作品"; }
		***/}),
		/*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
		 *
		 *   シリーズページ
		 *
		 *-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/
		//なろう＆18禁共通
		"ncode_series": (function () {/***
			#content .series_title::before {
				content:"NG作者作品"; color:white; background-color:red; display:inline-block;
				padding:2px 5px; margin-right:.5em; line-height:120%;
			}
		***/}),
	}
	//////////// 処理定義 ////////////
	const define = {
		"top": {
			"path": {
				"pickup": {
					"caption": "ピックアップリスト",
					"rule": /^pickup\/list\//,
					"func": [一覧系共通処理],
					"selector": ".trackback_list",
					"toggle": true,
					"css": ["pickup"],
				},
				"searchuser": {
					"caption": "ユーザ検索",
					"rule": /^searchuser\/search\//,
					"func": [一覧系共通処理],
					"selector": ".main2_box > div",
					"css": ["searchuser"],
				},
			}
		},
		"yomou": {
			"path": {
				"search": {
					"caption": "検索結果",
					"rule": /^search(?:\/search\/|\.php)/,
					"func": [一覧系共通処理, 検索結果用各種算出処理],
					"selector": ".searchkekka_box",
					"toggle": true,
					"css": ["search"],
				},
				"rankgenretop": {
					"caption": "ランキングトップ（ジャンル別）",
					"rule": /^rank\/(?:genre)top\//,
					"func": [ジャンル別ランキングトップ処理],
					"selector": ".genreranking_topbox > ul > li",
					"toggle": true,
					"css": ["rankgenretop"],
				},
				"ranktop": {
					"caption": "ランキングトップ（総合、異世界転生/転移）",
					"rule": /^rank\/(?:isekai)?top\//,
					"func": [一覧系共通処理],
					"selector": ".ranking_top5box > ul > li",
					"toggle": true,
					"css": ["ranktop"],
				},
				"ranklist": {
					"caption": "ランキング一覧",
					"rule": /^rank\/(?:[a-z]+)?list\//,
					"func": [ランキング処理],
					"selector": ".ranking_list",
					"toggle": true,
					"css": ["ranklist"],
				},
				"nopointlist": {
					"caption": "未評価作品一覧、未感想作品一覧",
					"rule": /^nolist\/no(?:point|impression)list\//,
					"func": [一覧系共通処理],
					"selector": ".no_list",
					"toggle": true,
					"css": ["nopointlist"],
				},
				"avgranklist": {
					"caption": "平均点順作者一覧",
					"rule": /^userlist\/avgranklist\//,
					"func": [一覧系共通処理],
					"selector": ".review",
					"css": ["avgranklist"],
				},
			}
		},
		"mypage": {
			"path": {
				"user": {
					"caption": "マイページトップ",
					"rule": /^(\d+)\/$/,
					"func": [マイページ共通処理],
					"selector": "#anotheruser_info > h1",
					"css": ["mypage"],
				},
				"blog": {
					"caption": "活動報告他",
					"rule": /^mypage(?:.+list|blog\/view)\/userid\/(\d+)\//,
					"func": [マイページ共通処理],
					"selector": "#anotheruser_info > h1",
					"css": ["mypage"],
				},
			}
		},
		"xmypage": {
			"path": {
				"user": {
					"caption": "Ｘマイページトップ",
					"rule": /^(x\d{4}[a-z]+)\/$/,
					"func": [Ｘマイページ処理],
					"selector": "#header > h1",
					"css": ["xmypage","xmypage_top"],
				},
				"novellist": {
					"caption": "Ｘ投稿作品一覧",
					"rule": /^mypage\/novellist\/xid\/(x\d{4}[a-z]+)\//,
					"func": [マイページ共通処理],
					"selector": "#header > h1",
					"css": ["xmypage"],
				},
				"blog": {
					"caption": "Ｘ活動報告他",
					"rule": /^mypage(?:.+list|blog\/view)\/xid\/(\d+)\//,
					"func": [Ｘユーザー処理],
					"selector": "#header > h1",
					"css": ["xmypage"],
				},
			}
		},
		"ncode": {
			"path": {
				"top": {
					"caption": "連載小説トップまたは短編",
					"rule": /^n\d{4}[a-z]+\/?$/,
					"func": [小説ページ処理],
					"css": ["ncode_top","ncode_top_content"],
				},
				"novel": {
					"caption": "連載小説",
					"rule": /^n\d{4}[a-z]+\/\d+/,
					"func": [小説ページ処理],
					"css": ["ncode_novel","ncode_novel_content"],
				},
				"infotop": {
					"caption": "小説情報",
					"rule": /^novelview\/infotop\/ncode\/n\d{4}[a-z]+\/?$/,
					"func": [小説ページ処理],
					"css": ["ncode_infotop","ncode_infotop_content"],
				},
				"series": {
					"caption": "シリーズ",
					"rule": /^s[\d]{4}[a-z]+\/?$/,
					"func": [シリーズページ処理],
					"css": ["ncode_series"],
				},
			}
		},
		"novel18": {
			"path": {
				"top": {
					"caption": "Ｘ連載小説トップまたは短編",
					"rule": /^n\d{4}[a-z]+\/?$/,
					"func": [小説ページ処理],
					"css": ["ncode_top","novel18_top_content"],
				},
				"novel": {
					"caption": "Ｘ連載小説",
					"rule": /^n\d{4}[a-z]+\/\d+/,
					"func": [小説ページ処理],
					"css": ["ncode_novel","novel18_novel_content"],
				},
				"infotop": {
					"caption": "Ｘ小説情報",
					"rule": /^novelview\/infotop\/ncode\/n\d{4}[a-z]+\/?$/,
					"func": [小説ページ処理],
					"css": ["ncode_infotop","novel18_infotop_content"],
				},
				"series": {
					"caption": "Ｘシリーズ",
					"rule": /^xs[\d]{4}[a-z]+\/?$/,
					"func": [シリーズページ処理],
					"css": ["ncode_series"],
				},
			}
		}
	};

	//////////// メイン処理 ////////////
	const getDefineByUrl = function() {
		const host = location.hostname.toLowerCase();
		const path = location.pathname.toLowerCase().replace(/^\//, "");
		const subdomain = host.match(/^([a-z\d]+)\.syosetu\.com/) ? RegExp.$1 : "top";
		const item = define[subdomain];
		if (item) {
			for (let kind in item.path) {
				const o = item.path[kind];
				if (path.match(o.rule)) return { def:o, param:RegExp.$1 };
			}
		}
	};
	const defContainer = getDefineByUrl();
	if (defContainer) {
		const def = defContainer.def;
		if (def.func && def.func.length) {
			writeLog(def.caption);
			for (let i=0, len=def.func.length ; i<len ; i++) {
				def.func[i](def, defContainer.param);
			}
		} else {
			writeLog("処理未登録");
		}
	} else {
		writeLog("対象外ページ");
	}
})();
